#include "idle.h"
#include "wayland.h"
#include "ext-idle-notify-v1.h"
#include "shell.h"


struct _DiyaIdleNotification
{
    DiyaShellObject parent;
    struct ext_idle_notification_v1* notification;
    guint timeout;
    gchar* str;
};

G_DEFINE_FINAL_TYPE(DiyaIdleNotification, diya_idle_notification, DIYA_TYPE_SHELL_OBJECT)

static void diya_idle_notification_finalize(GObject* object)
{
    (void) object;
    DiyaIdleNotification * self = DIYA_IDLE_NOTIFICATION(object);
    if(self->notification)
    {
        ext_idle_notification_v1_destroy(self->notification);
        self->notification = NULL;
    }
    g_debug("diya_idle_notification_finalize: %s", diya_object_to_string(object));
    if(self->str)
    {
        g_free(self->str);
        self->str = NULL;
    }
    G_OBJECT_CLASS(diya_idle_notification_parent_class)->finalize(object);
}

static void diya_idle_notification_dispose(GObject* object)
{
    (void) object;
    // DiyaIdleNotification * self = DIYA_IDLE_NOTIFICATION(object);
    g_debug("diya_idle_notification_dispose: %s", diya_object_to_string(object));
    G_OBJECT_CLASS(diya_idle_notification_parent_class)->dispose(object);
}

static void diya_idle_notification_init(DiyaIdleNotification * self)
{
    self->notification = NULL;
    self->str = NULL;
}

static const gchar* diya_idle_notification_to_string(DiyaObject* object)
{
    DiyaIdleNotification * self = DIYA_IDLE_NOTIFICATION(object);
    if(!self->str)
    {
        self->str = (gchar*) malloc(64);
        g_snprintf(self->str, 64, "DiyaIdleNotification for timeout: %d (ms)", self->timeout);
    }
    return self->str;
}

static void diya_idle_notification_class_init(DiyaIdleNotificationClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    DiyaObjectClass *base_class = DIYA_OBJECT_CLASS(class);

    gobject_class->dispose = diya_idle_notification_dispose;
    gobject_class->finalize = diya_idle_notification_finalize;
    base_class->to_string = diya_idle_notification_to_string;

    g_signal_new(DIYA_SIGNAL_IDLE_NOTIF_IDLE,
        DIYA_TYPE_IDLE_NOTIFICATION,
        G_SIGNAL_DETAILED |
            G_SIGNAL_ACTION |
            G_SIGNAL_RUN_FIRST,
        0,
        NULL,
        NULL,
        NULL,
        G_TYPE_NONE,
        0);

    g_signal_new(DIYA_SIGNAL_IDLE_NOTIF_RESUME,
        DIYA_TYPE_IDLE_NOTIFICATION,
        G_SIGNAL_DETAILED |
            G_SIGNAL_ACTION |
            G_SIGNAL_RUN_FIRST,
        0,
        NULL,
        NULL,
        NULL,
        G_TYPE_NONE,
        0);
}

void diya_idle(void *data, struct ext_idle_notification_v1 *notification)
{
    (void) notification;
    DiyaIdleNotification* self = data;
    g_signal_emit_by_name(self, DIYA_SIGNAL_IDLE_NOTIF_IDLE);

}

void diya_resume(void *data, struct ext_idle_notification_v1 *notification)
{
    (void) notification;
    DiyaIdleNotification* self = data;
    g_signal_emit_by_name(self, DIYA_SIGNAL_IDLE_NOTIF_RESUME);
}

static const struct ext_idle_notification_v1_listener g_idle_listener_impl = 
{
    .idled = diya_idle,
    .resumed = diya_resume
};

void diya_idle_manager_register(gpointer mngr, DiyaShell *shell)
{
    (void) mngr;
    (void) shell;
}

DiyaIdleNotification* diya_shell_get_idle_notification(DiyaShell* shell, guint timeout_ms)
{
    DiyaWayland *wayland = diya_shell_get_wayland(shell);
    struct ext_idle_notifier_v1* mngr = diya_wayland_get_idle_mngr(wayland);
    if(!mngr)
    {
        return NULL;
    }
    DiyaIdleNotification* self = g_object_new(DIYA_TYPE_IDLE_NOTIFICATION, "shell", shell, NULL);
    struct wl_seat *seat = diya_wayland_get_seat(wayland);
    self->timeout = timeout_ms;
    self->notification = ext_idle_notifier_v1_get_idle_notification(mngr, timeout_ms, seat);
    ext_idle_notification_v1_add_listener(self->notification, &g_idle_listener_impl, self);
    return self;
}


guint diya_shell_notification_get_timeout(DiyaIdleNotification* self)
{
    return self->timeout;
}